@@ -1,4 +1,4 @@
-## ---- include = FALSE---------------------------------------------------------
+## ----include = FALSE----------------------------------------------------------
 knitr::opts_chunk$set(
   collapse = TRUE,
   #dev="svg",
@@ -13,6 +13,7 @@ library(mets)
 
 ## -----------------------------------------------------------------------------
 library(mets)
+library(timereg)
 set.seed(1000) # to control output in simulatins for p-values below.
 
 ## -----------------------------------------------------------------------------
@@ -24,7 +25,7 @@ base1 <- base1cumhaz
 base4 <- base4cumhaz
 rr <- simRecurrent(200,base1,death.cumhaz=ddr)
 rr$x <- rnorm(nrow(rr)) 
-rr$strata <- floor((rr$id-0.01)/500)
+rr$strata <- floor((rr$id-0.01)/100)
 dlist(rr,.~id| id %in% c(1,7,9))
 
 ## -----------------------------------------------------------------------------
@@ -96,8 +97,12 @@ recEFF1 <- recurrentMarginalAIPCW(Event(start,stop,statusD)~cluster(id),data=rr,
 with( recEFF1, cbind(times,muP,semuP,muPAt,semuPAt,semuPAt/semuP))
 
 times <- 500*(1:10)
+###recEFF14 <- recurrentMarginalAIPCW(Event(start,stop,statusD)~cluster(id),data=rr,times=times,cens.code=0,
+###death.code=3,cause=1,augment.model=~Nt+Nt2+expNt+NtexpNt)
+###with(recEFF14,cbind(times,muP,semuP,muPAt,semuPAt,semuPAt/semuP))
+
 recEFF14 <- recurrentMarginalAIPCW(Event(start,stop,statusD)~cluster(id),data=rr,times=times,cens.code=0,
-death.code=3,cause=1,augment.model=~Nt+Nt2+expNt+NtexpNt)
+death.code=3,cause=1,augment.model=~Nt+I(Nt^2)+I(exp(-Nt))+ I( Nt*exp(-Nt)))
 with(recEFF14,cbind(times,muP,semuP,muPAt,semuPAt,semuPAt/semuP))
 
 bplot(out,se=TRUE,ylab="marginal mean",col=2)
@@ -114,16 +119,22 @@ for (t in times) {
 legend("bottomright",c("Eff-pred"),lty=1,col=c(1,3))
 
 ## -----------------------------------------------------------------------------
-rr <- mets:::simMarginalMeanCox(200,cens=3/5000,Lam1=base1,LamD=ddr,beta1=c(0.3,-0.3),
-				betad=c(-0.3,0.3))
-dtable(rr,~statusG+status+death,level=2,response=1)
+n <- 200
+X <- matrix(rbinom(n*2,1,0.5),n,2)
+colnames(X) <- paste("X",1:2,sep="")
+###
+r1 <- exp( X %*% c(0.3,-0.3))
+rd <- exp( X %*% c(0.3,-0.3))
+rc <- exp( X %*% c(0,0))
+fz <- NULL
+rr <- mets:::simGLcox(n,base1,ddr,var.z=0,r1=r1,rd=rd,rc=rc,fz,model="twostage",cens=3/5000) 
+rr <- cbind(rr,X[rr$id+1,])
+
+dtable(rr,~statusD+status+death,level=2,response=1)
 
 times <- seq(500,5000,by=500)
-rr <- transform(rr,statusD=status)
-rr <- dtransform(rr,statusD=2,death==1)
-dtable(rr,~statusD+status+death,level=2)
 recEFF1x <- recurrentMarginalAIPCW(Event(start,stop,statusD)~cluster(id),data=rr,times=times,
-				   cens.code=0,death.code=2,cause=1,augment.model=~X1+X2)
+				   cens.code=0,death.code=3,cause=1,augment.model=~X1+X2)
 with(recEFF1x, cbind(muP,muPA,muPAt,semuP,semuPA,semuPAt,semuPAt/semuP))
 
 xr <- phreg(Surv(start,stop,status==1)~cluster(id),data=rr)
@@ -132,17 +143,106 @@ out <- recurrentMarginal(xr,dr)
 mets::summaryTimeobject(out$times,out$mu,times=times,se.mu=out$se.mu)
 
 ## -----------------------------------------------------------------------------
-rr <- mets:::simMarginalMeanCox(200,cens=3/5000,Lam1=base1,LamD=ddr,beta1=c(0.3,-0.3),
-				betad=c(-0.3,0.3))
-dtable(rr,~statusG+status+death,level=2,response=1)
+n <- 100
+X <- matrix(rbinom(n*2,1,0.5),n,2)
+colnames(X) <- paste("X",1:2,sep="")
+###
+r1 <- exp( X %*% c(0.3,-0.3))
+rd <- exp( X %*% c(0.3,-0.3))
+rc <- exp( X %*% c(0,0))
+fz <- NULL
+rr <- mets:::simGLcox(n,base1,ddr,var.z=1,r1=r1,rd=rd,rc=rc,fz,cens=1/5000,type=2) 
+rr <- cbind(rr,X[rr$id+1,])
 
- out  <- recreg(Event(start,stop,statusG)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,cens.code=0)
- outs <- recreg(Event(start,stop,statusG)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,cens.code=0,
+ out  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,cens.code=0)
+ outs <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,cens.code=0,
 		cens.model=~strata(X1,X2))
-
  summary(out)$coef
  summary(outs)$coef
 
+ ## checking baseline
+ par(mfrow=c(1,1))
+ bplot(out)
+ bplot(outs,add=TRUE,col=2)
+ lines(scalecumhaz(base1,1),col=3,lwd=2)
+
+## -----------------------------------------------------------------------------
+ outipcw  <- recregIPCW(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,
+			cens.code=0,times=2000)
+ outipcws <- recregIPCW(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,
+		    cens.code=0,times=2000,cens.model=~strata(X1,X2))
+ summary(outipcw)$coef
+ summary(outipcws)$coef
+
+## -----------------------------------------------------------------------------
+ out  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=c(1,3),
+		death.code=3,cens.code=0)
+ summary(out)$coef
+
+## -----------------------------------------------------------------------------
+ rr$binf <- rbinom(nrow(rr),1,0.5) 
+ rr$statusDC <- rr$statusD
+ rr <- dtransform(rr,statusDC=4, statusD==3 & binf==0)
+ rr$weight <- 1
+ rr <- dtransform(rr,weight=2,statusDC==3)
+
+ outC  <- recreg(Event(start,stop,statusDC)~X1+X2+cluster(id),data=rr,cause=c(1,3),
+		 death.code=c(3,4),cens.code=0)
+ summary(outC)$coef
+
+ outCW  <- recreg(Event(start,stop,statusDC)~X1+X2+cluster(id),data=rr,cause=c(1,3),
+		  death.code=c(3,4),cens.code=0,wcomp=c(1,2))
+ summary(outCW)$coef
+
+ bplot(out,ylab="Mean composite")
+ bplot(outC,col=2,add=TRUE)
+ bplot(outCW,col=3,add=TRUE)
+
+## -----------------------------------------------------------------------------
+out  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,
+	        cens.code=0,cox.prep=TRUE)
+baseiid <- IIDbaseline.cifreg(out,time=3000)
+GLprediid(baseiid,rr[1:5,])
+
+## -----------------------------------------------------------------------------
+ outi  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,cens.code=0,
+		augment.model=~Nt+X1+X2)
+ summary(outi)$coef
+ outA  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,cens.code=0,
+		augment.model=~Nt+X1+X2,augment=outi$lindyn.augment)
+ summary(outA)$coef
+
+
+## -----------------------------------------------------------------------------
+ out  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,
+		cause=1,death.code=3,cens.code=0,cox.prep=TRUE)
+ outs <- phreg(Event(start,stop,statusD==3)~X1+X2+cluster(id),data=rr)
+
+ tsout <- twostageREC(outs,out,data=rr)
+ summary(tsout)
+
+## -----------------------------------------------------------------------------
+n <- 100
+X <- matrix(rbinom(n*2,1,0.5),n,2)
+colnames(X) <- paste("X",1:2,sep="")
+###
+r1 <- exp( X %*% c(0.3,-0.3))
+rd <- exp( X %*% c(0.3,-0.3))
+rc <- exp( X %*% c(0,0))
+rr <- mets:::simGLcox(n,base1,ddr,var.z=0,r1=r1,rd=rd,rc=rc,model="twostage",cens=3/5000) 
+rr <- cbind(rr,X[rr$id+1,])
+
+## -----------------------------------------------------------------------------
+rr <- mets:::simGLcox(100,base1,ddr,var.z=1,r1=r1,rd=rd,rc=rc,type=3,cens=3/5000) 
+rr <- cbind(rr,X[rr$id+1,])
+margsurv <- phreg(Surv(start,stop,statusD==3)~X1+X2+cluster(id),rr)
+recurrent <- phreg(Surv(start,stop,statusD==1)~X1+X2+cluster(id),rr)
+estimate(margsurv)
+estimate(recurrent)
+par(mfrow=c(1,2)); 
+plot(margsurv); lines(ddr,col=3); 
+plot(recurrent); lines(base1,col=3)
+
 ## -----------------------------------------------------------------------------
 ###cor.mat <- corM <- rbind(c(1.0, 0.6, 0.9), c(0.6, 1.0, 0.5), c(0.9, 0.5, 1.0))
 rr <- simRecurrentII(200,base1,base4,death.cumhaz=ddr,cens=3/5000,dependence=4,var.z=1)
@@ -194,7 +294,7 @@ plot(covrp)
 covrpS <- covarianceRecurrentS(rr,1,2,status="status",death="death",
         start="entry",stop="time",strata="strata",id="id",names.count="Count")
 
-## ---- eval=FALSE--------------------------------------------------------------
+## ----eval=FALSE---------------------------------------------------------------
 #  times <- seq(500,5000,500)
 #  
 #  coo1 <- covarianceRecurrent(rr,1,2,status="status",start="entry",stop="time")
@@ -208,7 +308,7 @@ covrpS <- covarianceRecurrentS(rr,1,2,status="status",death="death",
 #  cbind(times,mu2.1,mu2.i)
 #  cbind(times,mu1.2,mu1.i)
 
-## ---- eval=FALSE--------------------------------------------------------------
+## ----eval=FALSE---------------------------------------------------------------
 #  bt1 <- BootcovariancerecurrenceS(rr,1,2,status="status",start="entry",stop="time",K=100,times=times)
 #  #bt1 <- Bootcovariancerecurrence(rr,1,2,status="status",start="entry",stop="time",K=K,times=times)
 #  
@@ -224,21 +324,21 @@ covrpS <- covarianceRecurrentS(rr,1,2,status="status",death="death",
 #  	bse.dmu1.i=apply(bt1$mu1.2-bt1$mu1.i,1,sd)
 #  	)
 
-## ---- eval=FALSE--------------------------------------------------------------
+## ----eval=FALSE---------------------------------------------------------------
 #  tt  <- BCoutput$dmugi/BCoutput$bse.dmugi
 #  cbind(times,2*(1-pnorm(abs(tt))))
 
-## ---- eval=FALSE--------------------------------------------------------------
+## ----eval=FALSE---------------------------------------------------------------
 #  t21  <- BCoutput$dmu1.i/BCoutput$bse.dmu1.i
 #  t12  <- BCoutput$dmu2.i/BCoutput$bse.dmu2.i
 #  cbind(times,2*(1-pnorm(abs(t21))),2*(1-pnorm(abs(t12))))
 
-## ---- eval=FALSE--------------------------------------------------------------
+## ----eval=FALSE---------------------------------------------------------------
 #  par(mfrow=c(1,2))
 #  matplot(BCoutput$bt1$time,BCoutput$bt1$EN1N2,type="l",lwd=0.3)
 #  matplot(BCoutput$bt1$time,BCoutput$bt1$EIN1N2,type="l",lwd=0.3)
 
-## ---- eval=FALSE--------------------------------------------------------------
+## ----eval=FALSE---------------------------------------------------------------
 #    data(base1cumhaz)
 #    data(base4cumhaz)
 #    data(drcumhaz)
@@ -256,7 +356,7 @@ covrpS <- covarianceRecurrentS(rr,1,2,status="status",death="death",
 #    coo <- covarianceRecurrent(rr,1,2,status="status",start="entry",stop="time")
 #    plot(coo,main ="Scenario I")
 
-## ---- eval=FALSE--------------------------------------------------------------
+## ----eval=FALSE---------------------------------------------------------------
 #    var.z <- c(0.5,0.5,0.5)
 #    # death related to  both causes in same way
 #    cor.mat <- corM <- rbind(c(1.0, 0.0, 0.5), c(0.0, 1.0, 0.5), c(0.5, 0.5, 1.0))
@@ -266,7 +366,7 @@ covrpS <- covarianceRecurrentS(rr,1,2,status="status",death="death",
 #    par(mfrow=c(1,3))
 #    plot(coo,main ="Scenario II")
 
-## ---- eval=FALSE--------------------------------------------------------------
+## ----eval=FALSE---------------------------------------------------------------
 #    var.z <- c(0.5,0.5,0.5)
 #    # positive dependence for N1 and N2 all related in same way
 #    cor.mat <- corM <- rbind(c(1.0, 0.5, 0.5), c(0.5, 1.0, 0.5), c(0.5, 0.5, 1.0))
@@ -276,7 +376,7 @@ covrpS <- covarianceRecurrentS(rr,1,2,status="status",death="death",
 #    par(mfrow=c(1,3))
 #    plot(coo,main="Scenario III")
 
-## ---- eval=FALSE--------------------------------------------------------------
+## ----eval=FALSE---------------------------------------------------------------
 #    var.z <- c(0.5,0.5,0.5)
 #    # negative dependence for N1 and N2 all related in same way
 #    cor.mat <- corM <- rbind(c(1.0, -0.4, 0.5), c(-0.4, 1.0, 0.5), c(0.5, 0.5, 1.0))
